﻿using Microsoft.Win32;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading;

namespace Discrete_Registry_Processor
{
    class Program
    {
        static void Main(string[] args)
        {
            /* 
             * This program was created by Jared Kohr on 12/10/17.
             * 
             * Purpose: Some machines appear to have been imaged with registry entries that should have been handled by group policy.
             *          They are causing PCs to have an incorrect configuration resulting in service calls. Due to the key locations, 
             *          these bad configurations would replicate into every new user account extending the problem's existence.
             *          This program deletes the keys responsible and forces a group policy update.
             *          
             * Note: .DEFUALT is an alias for HKEY_USERS\S-1-5-18, so this does not need to be deleted or handled seperatly. Hives unload at PC reboot.
             * 
             * Process: (1) Since not all profiles are loaded in HKEY_USERS, try to load hives from
             *          HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\Windows NT\CurrentVersion\ProfileList into HKEY_USERS
             *          (2) When loading hives, delete {profile}\Software\Microsoft\Windows\Current Version\Internet Setings
             *          If an error occurs, ignore it.
             */

            RegistryKey key = Registry.LocalMachine.OpenSubKey(@"Software\Microsoft\Windows NT\CurrentVersion\ProfileList");

            foreach (var v in key.GetSubKeyNames())
            {
                Console.ForegroundColor = ConsoleColor.Blue;
                Console.WriteLine("Evaluating "+v+" ...\n");
                Console.ResetColor();

                RegistryKey vSubKey = key.OpenSubKey(v);
                if (vSubKey != null)
                {
                    foreach (var value in vSubKey.GetValueNames())
                    {
                        //Console.WriteLine(value);

                        //string productName = Convert.ToString(vSubKey.GetValue("DisplayName"));
                        try
                        {
                            if (value.ToString().Equals("ProfileImagePath", StringComparison.OrdinalIgnoreCase))
                            {
                                string profileID = Convert.ToString(v);
                                string path = Convert.ToString(vSubKey.GetValue("ProfileImagePath"));
                                try
                                {
                                    LoadHive(profileID, path);
                                }
                                catch (Exception e)
                                {
                                    Console.WriteLine("Unable to load hive. Error: "  + e +"\n");
                                }

                                try
                                {
                                    EditHive(profileID);
                                }
                                catch (Exception e)
                                {
                                    Console.WriteLine("Unable to edit hive. Error: "  + e +"\n");
                                }
                            }
                        }
                        catch (Exception e)
                        {
                            Console.WriteLine("Error: "  + e +"\n");
                        }
                    }

                }
            }
            key.Close();

            ApplyGroupPolicy();

            Console.ForegroundColor = ConsoleColor.Green;
            Console.WriteLine("All profile keys have been processed. Press [enter] to exit.");
            Console.ResetColor();
            Console.ReadLine();
        }

        public static void LoadHive(string profileID, string path)
        {
            Console.WriteLine("ProfileID: " + profileID + "\nPath: " + path +"\n");
            SilentExecute("reg.exe", "LOAD HKU\\" + profileID + " \"" + path + "\\NTUSER.DAT\"");
        }

        public static void EditHive(string profileID)
        {
            RegistryKey registrykeyHKU = Registry.Users;
            string keyPath = profileID+@"\Software\Microsoft\Windows\CurrentVersion\Internet Settings";

            try
            {
                registrykeyHKU.DeleteSubKeyTree(keyPath);
                Console.ForegroundColor = ConsoleColor.Green;
                Console.WriteLine("Deleted key:" + keyPath+"\n");
                Console.ResetColor();
            }
            catch (Exception e)
            {
                Console.ForegroundColor = ConsoleColor.Red;
                Console.WriteLine("Error:"  + e +"\n");
                Console.ResetColor();
            }

            registrykeyHKU.Close();
        }

        public static void ApplyGroupPolicy()
        {
            try
            {
                Console.ForegroundColor = ConsoleColor.Blue;
                Console.WriteLine("Attempting to update Group Policy ...\n");
                Console.ResetColor();
                SilentExecute("gpupdate.exe", "/force");
                Console.ForegroundColor = ConsoleColor.Yellow;
                Console.WriteLine("Group policy has been updated. A reboot may be necessary.\n");
                Console.ResetColor();
            }
            catch (Exception e)
            {
                Console.ForegroundColor = ConsoleColor.Red;
                Console.WriteLine("Group policy has failed to update. Error: "+e+"\n");
                Console.ResetColor();
            }
            
        }

        public static void SilentExecute(string executable, string args)
        {
            ProcessStartInfo psi = new ProcessStartInfo();
            psi.FileName = executable;
            psi.UseShellExecute = false;
            psi.RedirectStandardError = true;
            psi.RedirectStandardOutput = true;
            psi.Arguments = args;

            Process process = Process.Start(psi);
            process.WaitForExit();

            string errorOutput = process.StandardError.ReadToEnd();
            string standardOutput = process.StandardOutput.ReadToEnd();

            Console.ForegroundColor = ConsoleColor.Yellow;
            if (process.ExitCode != 0)
            {
                
                throw new Exception(executable + " exit code: " + process.ExitCode.ToString() + " " + (!string.IsNullOrEmpty(errorOutput) ? " " + errorOutput : "") + " " + (!string.IsNullOrEmpty(standardOutput) ? " " + standardOutput : ""));
            }
            Console.ResetColor();
        }
    }
}
